#! Rscript --vanilla
### Input: 1) Functions S_T (or other statistics to be compared, e.g. # of sig. genes in the fun category)
### 2) random sample x Functions S_t

### Final output file: Function\tp-value\tadj p-value\tALI p-value\tALI adj p-value\tAvgIntfungenes

Args <- commandArgs(TRUE)
if (length(Args)!=7){
        print("Usage: i) funST ; ii) randomsampleS_t ; iii) numfunintgenes ; iv) numfunintgenes_sample ; v) avgintfungenes_perconnectedgraph ; vi) avgintfungenes_perconnectedgraph_sample ; vii) output")
        quit()
}

#print(Args[1])
print("Calculate p-values....")
intgenes=as.matrix(read.table(Args[1],header=T,sep="\t",check.names=F))
randomgenes=as.matrix(read.table(Args[2],header=T,sep="\t",check.names=F))

numfunintgenes=as.matrix(read.table(Args[3],header=T,sep="\t",check.names=F))
numfunintgenes_sample=as.matrix(read.table(Args[4],header=T,sep="\t",check.names=F))

avgintfungenes_perconnectedgraph=as.matrix(read.table(Args[5],header=T,sep="\t",check.names=F))
avgintfungenes_perconnectedgraph_sample=as.matrix(read.table(Args[6],header=T,sep="\t",check.names=F))

uniqfuns <- colnames(intgenes)

### re-order
intgenes <- t(intgenes[,uniqfuns])
randomgenes <- randomgenes[,uniqfuns]

numfunintgenes <- t(numfunintgenes[,uniqfuns])
numfunintgenes_sample <- numfunintgenes_sample[,uniqfuns]

avgintfungenes_perconnectedgraph <- t(avgintfungenes_perconnectedgraph[,uniqfuns])
avgintfungenes_perconnectedgraph_sample <- avgintfungenes_perconnectedgraph_sample[,uniqfuns]


result <- matrix(numeric(),dim(intgenes)[2],6)
rownames(result) <- colnames(intgenes)
colnames(result) <- c("pval","adjpval","ALI_pval","ALI_adjpval","AvgIntFungenes_per_connectedgraph_pval","AvgIntFungenes_per_connectedgraph_adjpval")

### re-sampling only 5000
### rs <- 5000
### could also set rs to be the same number of samples in randomgenes, but would take longer time for resampling process
rs <- dim(randomgenes)[1]

tempp <- matrix(numeric(),rs,dim(randomgenes)[2])
tempp2 <- matrix(numeric(),rs,dim(randomgenes)[2])
tempp3 <- matrix(numeric(),rs,dim(randomgenes)[2])


### laplace smoothing to avoid p-val = 0

tempdim1 <- dim(randomgenes)[1]

#Sys.time()
for (j in 1:dim(tempp)[1]){
	k <- sample(1:tempdim1,1)	
	ks <- sample((1:tempdim1)[-k],tempdim1,replace=TRUE)
	tempintgenes <- t(randomgenes[k,])
	temprandomgenes <- randomgenes[ks,]

	tempnumfunintgenes <- t(numfunintgenes_sample[k,])
	tempnumfunintgenes_sample <- numfunintgenes_sample[ks,]

	tempavgintfungenes_perconnectedgraph <- t(avgintfungenes_perconnectedgraph_sample[k,])
	tempavgintfungenes_perconnectedgraph_sample <- avgintfungenes_perconnectedgraph_sample[ks,]

	tempint1 <- t(matrix(tempintgenes,length(uniqfuns),tempdim1))
	tempint2 <- t(matrix(tempnumfunintgenes,length(uniqfuns),tempdim1))
	tempint3 <- t(matrix(tempavgintfungenes_perconnectedgraph,length(uniqfuns),tempdim1))

	tempp[j,] <- (colSums(temprandomgenes>=tempint1)+1)/(dim(temprandomgenes)[1]+1)
	tempp2[j,] <- (colSums(tempnumfunintgenes_sample>=tempint2)+1)/(dim(tempnumfunintgenes_sample)[1]+1)
	tempp3[j,] <- (colSums(tempavgintfungenes_perconnectedgraph_sample>=tempint3)+1)/(dim(tempavgintfungenes_perconnectedgraph_sample)[1]+1)

}
#Sys.time()

tempp_minp <- apply(tempp,1,function(x){min(x)})
tempp2_minp <- apply(tempp2,1,function(x){min(x)})
tempp3_minp <- apply(tempp3,1,function(x){min(x)})

for (i in 1:dim(result)[1]){
	result[i,1] <- ((sum(as.numeric(randomgenes[,i])>=as.numeric(intgenes[1,i]))+1)/(length(randomgenes[,i])+1))
	result[i,2] <- ((sum(tempp_minp<=result[i,1])+1)/(length(tempp_minp)+1))

	result[i,3] <- ((sum(as.numeric(numfunintgenes_sample[,i])>=as.numeric(numfunintgenes[1,i]))+1)/(length(numfunintgenes_sample[,i])+1))
	result[i,4] <- ((sum(tempp2_minp<=result[i,3])+1)/(length(tempp2_minp)+1))

	result[i,5] <- ((sum(as.numeric(avgintfungenes_perconnectedgraph_sample[,i])>=as.numeric(avgintfungenes_perconnectedgraph[1,i]))+1)/(length(avgintfungenes_perconnectedgraph_sample[,i])+1))
	result[i,6] <- ((sum(tempp3_minp<=result[i,5])+1)/(length(tempp3_minp)+1))

}
#Sys.time()

write.table(result,file=Args[7],row.names=T,col.names=F,sep="\t",quote=F)
